<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Ajax Cart Rate Limiter by Amasty
 */

namespace Amasty\AjaxCartRateLimiter\Model;

use Amasty\Base\Model\ConfigProviderAbstract;

class ConfigProvider extends ConfigProviderAbstract
{
    /**
     * @var string
     */
    protected $pathPrefix = 'amasty_cart/';

    public const AJAX_CART_REQUESTS_LIMITATION_ENABLED = 'ajax_cart_requests_limitation/enabled';
    public const REQUESTS_LIMITATION_NUMBER = 'ajax_cart_requests_limitation/requests_number';
    public const REQUESTS_LIMITATION_PERIOD = 'ajax_cart_requests_limitation/limit_period';
    public const DEFAULT_CONFIG = [
        self::REQUESTS_LIMITATION_NUMBER => 100,
        self::REQUESTS_LIMITATION_PERIOD => 1
    ];

    public function isAjaxCartRequestsLimitationEnabled(): bool
    {
        return $this->isSetFlag(self::AJAX_CART_REQUESTS_LIMITATION_ENABLED);
    }

    public function getRequestLimitationNumber(): int
    {
        return (int)$this->getValue(self::REQUESTS_LIMITATION_NUMBER)
            ?: self::DEFAULT_CONFIG[self::REQUESTS_LIMITATION_NUMBER];
    }

    public function getRequestLimitationPeriod(): int
    {
        return (int)$this->getValue(self::REQUESTS_LIMITATION_PERIOD)
            ?: self::DEFAULT_CONFIG[self::REQUESTS_LIMITATION_PERIOD];
    }
}
