<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Ajax Cart Rate Limiter by Amasty
 */

namespace Amasty\AjaxCartRateLimiter\Plugin\Cart\Controller\Cart;

use Amasty\AjaxCartRateLimiter\Model\RateLimiter;
use Amasty\Cart\Controller\Cart\Add;
use Magento\Framework\Message\ManagerInterface;

class LockAdd
{
    /**
     * @var RateLimiter
     */
    private $rateLimiter;

    /**
     * @var ManagerInterface
     */
    private $messageManager;

    public function __construct(
        RateLimiter $rateLimiter,
        ManagerInterface $messageManager
    ) {
        $this->rateLimiter = $rateLimiter;
        $this->messageManager = $messageManager;
    }

    public function aroundExecute(Add $subject, callable $proceed): void
    {
        $rateLimiter = $this->rateLimiter->getLimiter(RateLimiter::AJAX_CART_LIMITER);

        if ($rateLimiter && !$rateLimiter->consume()->isAccepted()) {
            $subject->getResponse()->setStatusHeader(429, '1.1', 'Too Many Requests');
            $this->messageManager->addErrorMessage(__("Too Many Requests"));
        } else {
            $proceed();
        }
    }
}
